﻿'***************************************************************************************************
'   Excelへのデータ貼り付けテスト(起動フォーム)               dlgExcelKidouL(Form)
'
' ※Excelワークシートへのデータ貼り付けの方法による処理時間を計測してみます。
' 　データは単なる1からの連番ですが、100列×1000行で、つまり10万件です。
'   ４種類の方法は、
'   ・１セルずつ直接数値データを貼り付ける
'   ・１行ずつ一次配列変数に格納してから１行(100列)単位にデータを貼り付ける
'   ・100列×1000行の二次元配列にデータをセットしてから全体を一回で貼り付ける
'   ・一旦JAG配列に格納してから二次元配列に変換させて全体を一回で貼り付ける
'   となります。処理後に処理時間をメッセージ表示します。
'
' ※もう一つですが、このような一括系処理をWindowsフォームで行なう場合の問題で
'   処理中にオペレータが他のボタンをクリックしてしまうとか、フォーム自体を
'   閉じてしまう、といったことが起きる可能性があります。
'   そこでオペレータに処理中であることが視覚的に判るようにサブフォームを開いて
'   そのサブフォーム側で実際の処理を行なうようにしてみました。
'   サブフォームはモーダル(TopMost)にしてControlBox=Falseとして処理中は
'   閉じられないようにしてプログレスバーで処理進捗が判るようにしています。
'
'   作成者:井上治  URL:https://www.ne.jp/asahi/excel/inoue/ [Excelでお仕事!]
'***************************************************************************************************
' 変更日付 Rev   変更履歴内容---------------------------------------------------------------------->
' 09/06/21(1.0.0.0)新規作成
' 17/04/16(1.0.1.0)記述統制見直し、GP_SYORI_MODE4の追加
' 17/04/21(1.0.1.0)Excel関連クラス統合作業
'***************************************************************************************************
Public Class dlgExcelKidouL
    '===============================================================================================
    Private Const g_cnsTitle As String = "Excelへのデータ貼り付けテスト"
    Private ReadOnly g_tblSYORI_MODE() As String = {"", "①", "②", "③", "④"}

    '***********************************************************************************************
    '   ■■■ フォーム上のコントロールイベント ■■■
    '***********************************************************************************************
    '* 処理名　：Button1_Click
    '* 機能　　：Excelへのデータ貼り付けテスト①
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(既定)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2009年06月21日
    '* 作成者　：井上　治
    '* 更新日　：2009年06月21日
    '* 更新者　：井上　治
    '* 機能説明：※１セルずつ直接数値データを貼り付ける
    '* 注意事項：　⇒実際の処理はサブフォーム(dlgExcelSyori)で行なわれます。
    '***********************************************************************************************
    Private Sub Button1_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) Handles Button1.Click
        '-------------------------------------------------------------------------------------------
        ' サブフォーム(処理フォーム)の呼び出し
        Call GP_SHOW_SUBFORM(1)
    End Sub

    '***********************************************************************************************
    '* 処理名　：Button2_Click
    '* 機能　　：Excelへのデータ貼り付けテスト②
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(既定)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2009年06月21日
    '* 作成者　：井上　治
    '* 更新日　：2009年06月21日
    '* 更新者　：井上　治
    '* 機能説明：※１行ずつ一次配列変数に格納してから１行(100列)単位にデータを貼り付ける
    '* 注意事項：　⇒実際の処理はサブフォーム(dlgExcelSyori)で行なわれます。
    '***********************************************************************************************
    Private Sub Button2_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) Handles Button2.Click
        '-------------------------------------------------------------------------------------------
        ' サブフォーム(処理フォーム)の呼び出し
        Call GP_SHOW_SUBFORM(2)
    End Sub

    '***********************************************************************************************
    '* 処理名　：Button3_Click
    '* 機能　　：Excelへのデータ貼り付けテスト③
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(既定)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2009年06月21日
    '* 作成者　：井上　治
    '* 更新日　：2009年06月21日
    '* 更新者　：井上　治
    '* 機能説明：※100列×1000行の二次元配列にデータをセットしてから全体を一回で貼り付ける
    '* 注意事項：　⇒実際の処理はサブフォーム(dlgExcelSyori)で行なわれます。
    '***********************************************************************************************
    Private Sub Button3_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) Handles Button3.Click
        '-------------------------------------------------------------------------------------------
        ' サブフォーム(処理フォーム)の呼び出し
        Call GP_SHOW_SUBFORM(3)
    End Sub

    '***********************************************************************************************
    '* 処理名　：Button4_Click
    '* 機能　　：Excelへのデータ貼り付けテスト④
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：(既定)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2017年04月16日
    '* 作成者　：井上　治
    '* 更新日　：2017年04月16日
    '* 更新者　：井上　治
    '* 機能説明：※一旦JAG配列に格納してから二次元配列に変換させて全体を一回で貼り付ける
    '* 注意事項：　⇒実際の処理はサブフォーム(dlgExcelSyori)で行なわれます。
    '***********************************************************************************************
    Private Sub Button4_Click(ByVal sender As System.Object, _
                              ByVal e As System.EventArgs) Handles Button4.Click
        '-------------------------------------------------------------------------------------------
        ' サブフォーム(処理フォーム)の呼び出し
        Call GP_SHOW_SUBFORM(4)
    End Sub

    '***********************************************************************************************
    ' ■■■ 共通サブ処理 ■■■
    '***********************************************************************************************
    '* 処理名　：GP_SHOW_SUBFORM
    '* 機能　　：サブフォーム(処理フォーム)の呼び出し
    '-----------------------------------------------------------------------------------------------
    '* 返り値　：(なし)
    '* 引数　　：Arg1 = 処理モード値(Integer)
    '-----------------------------------------------------------------------------------------------
    '* 作成日　：2009年06月21日
    '* 作成者　：井上　治
    '* 更新日　：2017年04月21日
    '* 更新者　：井上　治
    '* 機能説明：
    '* 注意事項：
    '***********************************************************************************************
    Private Sub GP_SHOW_SUBFORM(ByVal intSYORI_MODE As Integer)
        '-------------------------------------------------------------------------------------------
        Dim strTitle As String = g_cnsTitle & g_tblSYORI_MODE(intSYORI_MODE) ' 処理タイトル
        Dim tspJIKAN As TimeSpan                                    ' 処理時間(TimeSpan)
        Using objSyoriForm As New dlgExcelSyoriL
            With objSyoriForm
                .prpSYORI_MODE = intSYORI_MODE
                .ShowDialog(Me)
                tspJIKAN = .prpSYORI_JIKAN
            End With
        End Using
        Dim strMSG As String = "処理時間は、" & tspJIKAN.TotalSeconds.ToString & "秒です。" ' MSG
        MessageBox2.DialogShow(Me, _
                               strMSG, _
                               strTitle, _
                               MessageBoxButtons.OK, _
                               MessageBoxIcon.Information)
    End Sub

    '----------------------------------------<< End of Source >>------------------------------------
End Class
